//------------------------------------------------------------------------
#define VENDORID    0x03eb
#define PRODUCTID   0x7a53

#define cmdEcho 0
#define cmdLCD_Init 1
#define cmdLCD_Command 2
#define cmdLCD_Data 3
#define cmdLCD_BG 4

#define msgOK 0
#define msgErr 1

#include <windows.h>
#include <usb.h>

short _stdcall USB_LCD_Test (void);
short _stdcall USB_LCD_Init (short increase, short shift, short C_on, short B_on, 
							 short D_shift, short R_shift, short lines, short font, short LCD);
short _stdcall USB_LCD_Command (unsigned char command, short LCD);
short _stdcall USB_LCD_Data (char data[], short len, short LCD);
short _stdcall USB_LCD_BL (short value);

//------------------------------------------------------------------------
// Die folgende Funktion unverndert belassen

BOOL WINAPI DllEntryPoint ( HINSTANCE hDLL, DWORD dwREASON, LPVOID Reserved )
{
    switch (dwREASON)
    { 
        case DLL_PROCESS_ATTACH: { break; }		
        case DLL_PROCESS_DETACH: { break; } 		 
    }
    return TRUE;
}

//------------------------------------------------------------------------

//struct usb_bus *usb_bus;
//struct usb_device *dev;
struct usb_device *usb_dev;
struct usb_dev_handle *usb_handle;

struct usb_device *device_init( unsigned int vendorid, unsigned int productid )
{
    struct usb_bus *usb_bus;
    struct usb_device *dev;
	
	/* Config USB trace debug, 0: None, 1-255: Trace depth */ 
    usb_set_debug( 0 );

    /* Initialize USB library */
    usb_init( );

    /* Generic USB library bus structure traversal */
    usb_find_busses( );
    usb_find_devices( );

    /* Scan device tree for vendor ID/product ID match */
    for ( usb_bus = usb_get_busses( ); usb_bus; usb_bus = usb_bus -> next )
    {
        for ( dev = usb_bus -> devices; dev; dev = dev -> next )
        {
            if (( dev->descriptor.idVendor == vendorid ) &&
                ( dev->descriptor.idProduct == productid ) )
                    return dev;
        }
    }

    return NULL;
}

short USB_SendBuffer(unsigned char request, short value, short index, char *buffer, short length){
    //struct usb_device *usb_dev;
    //struct usb_dev_handle *usb_handle;
	short ret;

    if ( usb_dev == NULL )
        return -1;

	/* Befehl ausfhren */
    usb_handle = usb_open( usb_dev );
    if ( usb_handle != NULL ){
	    ret = usb_control_msg(usb_handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | 0x00, request, value, index, buffer, length, 5000);
		usb_close( usb_handle );
	}else
		return -1;
	return ret;
}

short USB_ReceiveBuffer(unsigned char request, short value, short index, char *buffer, short length){
    //struct usb_device *usb_dev;
    //struct usb_dev_handle *usb_handle;
	short ret;

    /* Make initial connection 
    usb_dev = device_init( VENDORID, PRODUCTID );*/

    if ( usb_dev == NULL )
        return -1;

	/* Befehl ausfhren */
    usb_handle = usb_open( usb_dev );
    if ( usb_handle != NULL ){
	    ret = usb_control_msg(usb_handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | 0x80, request, value, index, buffer, length, 5000);
		usb_close( usb_handle );
	}else
		return -1;
	return ret;
}

//------------------------------------------------------------------------

short _stdcall USB_LCD_Connect (void){
	char buffer[8];
	
    /* Make initial connection */
    usb_dev = device_init( VENDORID, PRODUCTID );

	if (USB_ReceiveBuffer(cmdEcho, 123, 0, buffer, sizeof(buffer)) > 1)
		return (buffer[1] == 123);
	return 0;
}

short _stdcall USB_LCD_Init (short increase, short shift, short C_on, short B_on, 
							 short D_shift, short R_shift, short lines, short font, short LCD){
	char buffer[8];
	short command;

	increase &= 1;
	shift &= 1;
	C_on &= 1;
	B_on &= 1;
	D_shift &= 1;
	R_shift &= 1;
	lines &= 1;
	font &= 1;
	
	command = (increase << 0) | (shift << 1) | (C_on << 2) | (B_on << 3) | (D_shift << 4) 
		| (R_shift << 5) | (lines << 6) | (font << 7);

	if (USB_ReceiveBuffer(cmdLCD_Init, command, LCD, buffer, sizeof(buffer)) > 0)
		return (buffer[0] == msgOK);
	return 0;
}

short _stdcall USB_LCD_Command (unsigned char command, short LCD){
	char buffer[8];
	
	if (USB_ReceiveBuffer(cmdLCD_Command, command, LCD, buffer, sizeof(buffer)) > 0)
		return (buffer[0] == msgOK);
	return 0;
}

short _stdcall USB_LCD_Data (char data[], short len, short LCD){
	return (USB_SendBuffer(cmdLCD_Data, 0, LCD, data, len) == len);
}

short _stdcall USB_LCD_BL (short value){
	char buffer[8];
	
	if (USB_ReceiveBuffer(cmdLCD_BG, value, 0, buffer, sizeof(buffer)) > 0)
		return (buffer[0] == msgOK);
	return 0;
}

//------------------------------------------------------------------------

