#define msgOK 0
#define msgErr 1

#define cmdEcho 0
#define cmdLCD_Init 1
#define cmdLCD_Command 2
#define cmdLCD_Data 3
#define cmdLCD_BG 4

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/delay.h>
#include "usbdrv.h"

typedef struct {
	uchar increase:1;
	uchar shift:1;
	uchar C_on:1;
	uchar B_on:1;
	uchar D_shift:1;
	uchar R_shift:1;
	uchar lines:1;
	uchar font:1;
} LCD_Options_t;

static uchar replybuffer[2];
static uchar LCD_Nr;


//---------------------------------------------------------------

void LCD_Command(uchar command, uchar LCD){
	uchar enablemask;
	
	if (LCD == 2)
		enablemask = (1 << 5);
	else
		enablemask = (1 << 4);
	PORTB = command;
	PORTD &= ~(1 << 6);
	PORTD  |= enablemask;
	_delay_us(1);
	PORTD &= ~enablemask;
}

void LCD_Data(uchar data, uchar LCD){
	uchar enablemask;
	
	if (LCD == 2)
		enablemask = (1 << 5);
	else
		enablemask = (1 << 4);
	PORTB = data;
	PORTD |= (1 << 6);
	PORTD  |= enablemask;
	_delay_us(1);
	PORTD &= ~enablemask;
}

void LCD_Init(LCD_Options_t *Options, uchar LCD){
	uchar command;

	LCD_Command(0x30,LCD);
	_delay_ms(5);
	LCD_Command(0x30,LCD);
	_delay_us(50);
	_delay_us(50);
	LCD_Command(0x30,LCD);
	_delay_us(50);
	_delay_us(50);
	command = 0x30;
	if (Options->lines > 0)
		command |= (1 << 3);
	if (Options->font > 0)
		command |= (1 << 2);
	LCD_Command(command, LCD);
	_delay_us(50);
	LCD_Command(0x01, LCD);
	_delay_ms(2);
	command = 0x04;
	if (Options->increase > 0)
		command |= (1 << 1);
	if (Options->shift > 0)
		command |= 1;
	_delay_us(50);
	command = 0x10;
	if (Options->D_shift > 0)
		command |= (1 << 3);
	if (Options->R_shift > 0)
		command |= (1 << 2);
	LCD_Command(command, LCD);
	_delay_us(50);
	command = 0x0C;
	if (Options->C_on > 0)
		command |= (1 << 1);
	if (Options->B_on > 0)
		command |= 1;
	LCD_Command(command, LCD);
	_delay_us(50);
}

//---------------------------------------------------------------

uchar usbFunctionWrite(uchar *data, uchar len){
	uchar i;
	
	for (i=0;i<len;i++){
		LCD_Data(data[i],LCD_Nr);
		_delay_us(50);
	}
	return 0;
}

uchar	usbFunctionSetup(uchar data[8]){
	uchar len;

	replybuffer[0] = msgOK;
	switch (data[1]) {
		case cmdEcho:
			replybuffer[1] = data[2];
			len = 2;
			break;
		case cmdLCD_Init:
			LCD_Init((LCD_Options_t*)&data[2],data[4]);
			len = 1;
			break;
		case cmdLCD_Command:
			LCD_Command(data[2],data[4]);
			_delay_us(50);
			len = 1;
			break;
		case cmdLCD_Data:
			LCD_Nr = data[4];
			len = 0xFF;
			break;
		case cmdLCD_BG:
			if(data[2] != 0)
				PORTD |= (1 << 3);
			else
				PORTD &= ~(1 << 3);
			len = 1;
			break;
		default:
			replybuffer[0] = msgErr;
			len = 1;
			break;
	}
	usbMsgPtr = replybuffer;
	return len;
}

int	main(void){
	PORTB = 0xFF;
	DDRB = 0xFF;
	PORTD = 0;
	DDRD = 0b01111000;
	usbInit();
	sei();
	for(;;){	/* main event loop */
		usbPoll();
	}
	return 0;
}

//---------------------------------------------------------------

